function RunWaveStat(TDir)

% RunWaveStat('48k')

% In directory TDir, in each subdirectory, a file Stats.txt is stored in
% each subdirectory which gives the file by file statistics in that
% subdirectory.  A summary Stats.txt is stored in the top level directory.

% Read the list of file names of the form CA/CA01_01.wav
% The list must group files in a particular directory together

% $Id: RunWaveStat.m,v 1.1 2018/11/22 23:51:55 pkabal Exp $

FileList = readStrings('FileList.txt');
NL = length(FileList);

TDir = fullfile('..', TDir);
SDirP = '';
fid = [];
for i = 1:NL

  % See if we are finished with the current subdirectory
  SDir = fileparts(FileList{i});
  if (~strcmp(SDir, SDirP))  % Changing subdirectories
    if (~isempty(fid))
      fclose(fid);
    end

    % Open a new Stats.txt file in the new subdirectory
    fprintf('Processing subdirectory %s\n', SDir);
    SFile = fullfile(TDir, SDir, 'Stats.txt');
    fid = fopen(SFile, 'w');
    if (fid == -1)
      error('Error opening: %s', SFile);
    end
    SDirP = SDir;
  end

  % Get the statistics for the current file
  AFile = fullfile(TDir, FileList{i});
  Stat(i) = WaveStat(AFile);
  PrintFileStat(fid, Stat(i));
end
fclose(fid);

% Print accumulated statistics
TFile = fullfile(TDir, 'Stats.txt');
[~, DB] = fileparts(TDir);
fid = fopen(TFile, 'w');
PrintAccStat(fid, DB, Stat);
fclose(fid);

end

% --------------------
function CA = readStrings(Fname)

fid = fopen(Fname);
N = 0;
while (1)
  str = fgetl(fid);
  if (~ischar(str))
    break;
  end
  N = N + 1;
  CA{N} = str;
end
fclose(fid);

end

% --------------------
function Stat = WaveStat(Fname)

Amax = 32768;

[x, Fs] = audioread(Fname);
x = Amax * x;

Stat.Fname = Fname;
Stat.Sfreq = Fs;
Stat.Nframe = size(x, 1);
Stat.Nchan = size(x, 2);

% Standard statistics
Stat.Min = min(x);
Stat.Mean = mean(x);
Stat.Max = max(x);
Stat.SD = std(x);

% Active speech level
[Stat.AL, Stat.AF] = ActiveLevel(x, Stat.Sfreq);

end

% --------------------
function PrintFileStat(fid, Stat)

[~, FNS] = fileparts(Stat.Fname);
fprintf(fid, 'File: %s\n', FNS);
if (Stat.Nchan > 1)
  fprintf(fid, '  Number of frames  : %d\n', Stat.Nframe);
else
  fprintf(fid, '  Number of samples : %d\n', Stat.Nframe);
end
fprintf(fid, '  Std Dev = %-11.5g Mean = %.5g\n', Stat.SD, Stat.Mean);
fprintf(fid, '  Max     = %-11.5g Min  = %.5g\n', Stat.Max, Stat.Min);
fprintf(fid, '  Active Level: %.1f (Activity Factor: %.1f%%)\n', ...
	      Stat.AL, 100 * Stat.AF);

end
% --------------------
function PrintAccStat(fid, DB, Stat)

NL = length(Stat);

% Accumulated statistics
MinNframe = min([Stat.Nframe]);
MaxNframe = max([Stat.Nframe]);
ANframe   = sum([Stat.Nframe]) / NL;
MinMin    = min([Stat.Min]);
MaxMin    = max([Stat.Min]);
AMin      = sum([Stat.Min]) / NL;
MinMax    = min([Stat.Max]);
MaxMax    = max([Stat.Max]);
AMax      = sum([Stat.Max]) / NL;
MinMean   = min([Stat.Mean]);
MaxMean   = max([Stat.Mean]);
AMean     = sum([Stat.Mean]) / NL;
MinSD     = min([Stat.SD]);
MaxSD     = max([Stat.SD]);
ASD       = sum([Stat.SD]) / NL;
MinAL     = min([Stat.AL]);
MaxAL     = max([Stat.AL]);
AAL       = sum([Stat.AL]) / NL;
MinAF     = min([Stat.AF]);
MaxAF     = max([Stat.AF]);
AAF       = sum([Stat.AF]) / NL;

fprintf(fid, 'Average Statistics - Database: %s\n', DB);
fprintf(fid, '  Number of files: %d\n', NL);
fprintf(fid, '           [Min Avg Max]\n');
fprintf(fid, '  Number of samples: [%d %.6g %d]\n', ...
	      MinNframe, ANframe, MaxNframe);
fprintf(fid, '  Std Dev: [%.5g %.5g %.5g]\n', MinSD, ASD, MaxSD);
fprintf(fid, '  Mean   : [%.5g %.5g %.5g]\n', MinMean, AMean, MaxMean);
fprintf(fid, '  Min    : [%.5g %.5g %.5g]\n', MinMin, AMin, MaxMin);
fprintf(fid, '  Max    : [%.5g %.5g %.5g]\n',	MinMax, AMax, MaxMax);
fprintf(fid, '  Active Level   : [%.1f %.1f %.1f]\n', MinAL, AAL, MaxAL);
fprintf(fid, '  Activity Factor: [%.1f%% %.1f%% %.1f%%]\n', ...
	      100 * MinAF, 100 * AAF, 100 * MaxAF);

end
